"""Agent implementations for EVOSEAL.

This module contains various agent implementations that can be used with EVOSEAL
for different tasks and workflows.
"""

from evoseal.agents.agentic_system import AgenticSystem
from evoseal.agents.agentic_workflow_agent import WorkflowAgent

__all__ = ["AgenticSystem", "WorkflowAgent"]
