from setuptools import find_packages, setup

setup(
    name="my_evoseal_project",
    version="0.1.0",
    packages=find_packages(),
    install_requires=[
        # Add your project dependencies here
    ],
    python_requires=">=3.9",
)
