"""Version information for the EVOSEAL package."""

__version__ = "0.3.4"
__version_info__ = tuple(int(i) for i in __version__.split(".") if i.isdigit())

__all__ = ["__version__", "__version_info__"]
