"""
Knowledge module for the SEAL system.

This module provides the KnowledgeBase class for storing and retrieving
knowledge in the SEAL system.
"""

from .knowledge_base import KnowledgeBase

__all__ = ["KnowledgeBase"]
